// -------------------------------------------------------------
// O[oϐ
// -------------------------------------------------------------
float4x4 wvp                : WorldViewProjection;
float4x4 world_view         : WorldView;
float4x4 projection         : Projection;
float4x4 bones[12]          : Bone;
float4 material_diffuse     : Diffuse;
float4 material_ambient     : Ambient;
float4 material_specular    : Specular;
float  material_power       : SpecularPower;
float4 light_ambient        : LightAmbient;
float4 light_direction[4] 	: LightDirection;
float4 light_color[4]		: LightDiffuse;
float4 half_vector[1] 		: LightEyeHalf;
float4 eye_pos              : EyePosition;
float4 eye_dir		        : EyeDirection;
float4 object_color         : ObjectColor;
texture tex0                : Texture;
int num_bone_affect         : NumBonesAffect;
float4 userdata_specular_color;

sampler samp = sampler_state
{
    Texture = (tex0);
};

texture tex1 <string name="data\\tone.dds";>;
sampler samp1 = sampler_state
{
    Texture = (tex1);
};

// -------------------------------------------------------------
// _VF[_sNZVF[_ɓnf[^
// -------------------------------------------------------------
struct VSOutput
{
    float4 pos  : POSITION;
    float4 col  : COLOR0;
    float2 tex  : TEXCOORD0;
    float2 texB : TEXCOORD1;
};
VSOutput VS_Model(
    float4 pos			    : POSITION,		// f̒_
    float4 blend_weights	: BLENDWEIGHT,  // ufBȌd݃f[^
    float4 blend_indices    : BLENDINDICES, // ufBO CfbNX f[^
    float3 normal		    : NORMAL,       // f̖@
    float2 tex			    : TEXCOORD0,    // eNX`W
    uniform int num_bones                   // _ɉe{[̐
){
    VSOutput output = (VSOutput)0;

    float4 pos_blended = 0.0f;
    float3 normal_blended = 0.0f;

    if(num_bones>0)
    {
        // Compensate for lack of UBYTE4 on Geforce3
        int4 index_vector = D3DCOLORtoUBYTE4(blend_indices);

        // cast the vectors to arrays for use in the for loop below
        float blend_weight_array[4] = (float[4])blend_weights;
        int   index_array[4]        = (int[4])index_vector;

        float  last_weight = 0.0f;

        // calculate the pos/normal using the "normal" weights
        //        and accumulate the weights to calculate the last weight
        for( int i=0 ; i<num_bones-1 ; i++ )
        {
            last_weight = last_weight + blend_weight_array[i];

            pos_blended += mul(pos, bones[index_array[i]]) * blend_weight_array[i];
            normal_blended += mul(normal, bones[index_array[i]]) * blend_weight_array[i];
        }
        last_weight = 1.0f - last_weight;

        // Now that we have the calculated weight, add in the final influence
        pos_blended += (mul(pos, bones[index_array[num_bones-1]]) * last_weight);
        normal_blended += (mul(normal, bones[index_array[num_bones-1]]) * last_weight);

        pos_blended.w = 1.0f;
    }
    else
    {
        pos_blended = pos;
        normal_blended = normal;
    }

    output.pos = mul(pos_blended, wvp);

	// ^@͒PʃxNgł͂Ȃꍇ̂ŐKKv
	normal_blended = normalize(normal_blended);

    // ɂȂȂ悤ɂ
    float light_dot = max( dot( light_direction[0], normal_blended ), 0.0 );
	float4 light_power = light_dot * light_color[0] + light_ambient;
	light_power = min(light_power,1.0);

    output.col = object_color;
    tex.x /= 2.0f;
    output.tex = tex;
    output.texB = (0.5f, light_power);

    return output;
}

float4 PS(
    float4 col  : COLOR0,
    float2 tex  : TEXCOORD0,
    float2 texB : TEXCOORD1
     ) : COLOR
{
    float4 bcol = tex2D( samp1, texB );
    float4 c1 = tex2D( samp, tex ) * bcol.r;
    tex.x += 0.5f;
    float4 c2 = tex2D( samp, tex ) * (1.0f - bcol.r);
    return c1 + c2;
}

// -------------------------------------------------------------
// eNjbN
// -------------------------------------------------------------
VertexShader VS_Model_Array[5] =
{
    compile vs_2_0 VS_Model(0),
    compile vs_2_0 VS_Model(1),
    compile vs_2_0 VS_Model(2),
    compile vs_2_0 VS_Model(3),
    compile vs_2_0 VS_Model(4)
};

technique TechAdd
{
    pass P0
    {
        ZEnable = true;
        ZWriteEnable = false;
        AlphaBlendEnable = TRUE;
        DestBlend        = One;  
        
        VertexShader = (VS_Model_Array[num_bone_affect]);
        PixelShader = compile ps_2_0 PS();
    }
}

technique TechNormal
{
    pass P0
    {
        ZEnable = true;
        ZWriteEnable = true;
        AlphaBlendEnable = TRUE;
        
        VertexShader = (VS_Model_Array[num_bone_affect]);
        PixelShader = compile ps_2_0 PS();
    }
}

technique TechAlpha
{
    pass P0
    {
        ZEnable = true;
        ZWriteEnable = False;
        AlphaBlendEnable = TRUE;
        SrcBlend         = SRCALPHA;
        DestBlend        = InvSrcAlpha;  
        
        VertexShader = (VS_Model_Array[num_bone_affect]);
        PixelShader = compile ps_2_0 PS();
    }
}

